""" discover and run doctests in Cython extension modules."""
from __future__ import absolute_import

import keyword
import re
import sys
import tokenize
import pytest

try:
    import sysconfig
except ImportError:
    from distutils import sysconfig

import _pytest
from _pytest.doctest import get_optionflags
from _pytest.doctest import DoctestItem

try:
    from _pytest.doctest import _get_checker
except ImportError:
    _get_checker = None


def pytest_addoption(parser):
    group = parser.getgroup("cython")
    group.addoption(
        "--doctest-cython",
        action="store_true",
        default=False,
        help="run doctests in all .so and .pyd modules",
        dest="doctest_cython",
        )
    group.addoption(
        "--cython-ignore-import-errors",
        action="store_true",
        default=False,
        help="ignore doctest ImportErrors",
        dest="doctest_ignore_import_errors",
        )


def _find_matching_pyx_file(path, extensions):
    for ext in extensions:
        newpath = path.new(ext=ext)
        if newpath.check():
            return newpath


def pytest_collect_file(path, parent):
    bin_exts = ['.so']
    cy_exts = ['.pyx', '.py']  # collect .so files if .py file exists
    ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")

    config = parent.config
    if path.ext in bin_exts:
        if config.getoption('--doctest-cython'):
            if ext_suffix is None:
                bin_file = path
                # XXX EXT_SUFFIX is None for pypy (python2.7)
                if '.pypy' in path.basename:
                    basename = path.basename.split('.')[0]
                    bin_file = path.new(purebasename=basename, ext=path.ext)

            else:
                basename = path.basename.replace(ext_suffix, "")
                bin_file = path.new(purebasename=basename, ext=path.ext)

            pyx_file = _find_matching_pyx_file(bin_file, cy_exts)
            # only run test if matching .so and .pyx files exist
            # create addoption for this ??
            if pyx_file is not None:
                if hasattr(DoctestModule, 'from_parent'):
                    return DoctestModule.from_parent(parent, fspath=path)
                else:
                    # Backwards-compat for older pytest
                    return DoctestModule(path, parent)


# XXX if python2 support is dropped just use str.isidentifier
def _isidentifier(s):
    return (re.match('^' + tokenize.Name + '$', s)
            and not keyword.iskeyword(s))


# XXX copied from pytest but modified to use py.path instead; if Python 2
# support is dropped and support for more modern pytest added we can just use
# the one from pytest
def _resolve_package_path(path):
    """Return the Python package path by looking for the last
    directory upwards which still contains an __init__.py.

    Returns None if it can not be determined.
    """
    result = None
    for parent in path.parts(reverse=True):
        if parent.isdir():
            if not parent.join('__init__.py').isfile():
                break
            if not _isidentifier(parent.basename):
                break
            result = parent
    return result


# XXX patch pyimport to support PEP 3149
def _patch_pyimport(fspath, **kwargs):
    ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
    # XXX EXT_SUFFIX is None for pypy (python2.7)
    if ext_suffix is None and '.pypy' not in fspath.basename:
        return fspath.pyimport(**kwargs)

    else:
        # XXX EXT_SUFFIX is None for pypy (python2.7)
        if '.pypy' in fspath.basename:
            ext_suffix = fspath.ext
            basename = fspath.basename.split('.')[0]
            fspath = fspath.new(purebasename=basename, ext=fspath.ext)

        pkg_path = _resolve_package_path(fspath)
        if pkg_path is not None:
            pkg_root = pkg_path.dirname
            names = fspath.relto(pkg_root).split(fspath.sep)
            if names[-1].startswith('__init__.'):
                names.pop()
            module_name = '.'.join(names).replace(ext_suffix, '')
        else:
            pkg_root = fspath.parent
            module_name = fspath.basename.replace(ext_suffix, '')

        fspath._ensuresyspath(True, pkg_root)
        __import__(module_name)
        return sys.modules[module_name]


class DoctestModule(pytest.Module):

    def collect(self):
        import doctest

        if self.fspath.basename == "conftest.py":
            module = self.config.pluginmanager._importconftest(self.fspath)
        else:
            try:
                # XXX patch pyimport in pytest._pytest.doctest.DoctestModule
                module = _patch_pyimport(self.fspath)
            except ImportError:
                if self.config.getoption('--cython-ignore-import-errors'):
                    pytest.skip('unable to import module %r' % self.fspath)
                else:
                    raise

        # uses internal doctest module parsing mechanism
        finder = doctest.DocTestFinder()
        optionflags = get_optionflags(self)
        checker = None if _get_checker is None else _get_checker()
        runner = doctest.DebugRunner(verbose=0, optionflags=optionflags,
                                     checker=checker)
        for test in finder.find(module, module.__name__):
            if test.examples:  # skip empty doctests
                if hasattr(DoctestItem, 'from_parent'):
                    yield DoctestItem.from_parent(self, name=test.name,
                                                  runner=runner, dtest=test)
                else:
                    # Backwards-compat for older pytest
                    yield DoctestItem(test.name, self, runner, test)

    def _importtestmodule(self):
        # we assume we are only called once per module
        importmode = self.config.getoption("--import-mode", default=True)
        try:
            # XXX patch pyimport in pytest._pytest.pythod.Module
            mod = _patch_pyimport(self.fspath, ensuresyspath=importmode)
        except SyntaxError:
            raise self.CollectError(
                _pytest._code.ExceptionInfo().getrepr(style="short"))
        except self.fspath.ImportMismatchError:
            e = sys.exc_info()[1]
            raise self.CollectError(
                "import file mismatch:\n"
                "imported module %r has this __file__ attribute:\n"
                "  %s\n"
                "which is not the same as the test file we want to collect:\n"
                "  %s\n"
                "HINT: remove __pycache__ / .pyc files and/or use a "
                "unique basename for your test file modules"
                % e.args
            )
        # print "imported test module", mod
        self.config.pluginmanager.consider_module(mod)
        return mod



from __future__ import absolute_import

import py
import sys

import pytest_cython.plugin

PATH = py.path.local(__file__).dirpath()
PATH = PATH.join('example-project', 'src', 'pypackage')
sys.path.insert(0, str(PATH))


def test_cython_ext_module(testdir):
    module = PATH.listdir(fil='cython_ext_module*.so')[0]
    assert module.check()
    result = testdir.runpytest('-vv', '--doctest-cython', str(module))
    result.stdout.fnmatch_lines([
        "*Eggs.__init__ *PASSED*",
        "*Eggs.blarg*PASSED*",
        "*Eggs.fubar*PASSED*",
    ])
    assert result.ret == 0


def test_wrap_c_ext_module(testdir):
    module = PATH.listdir(fil='wrap_c_ext_module*.so')[0]
    assert module.check()
    result = testdir.runpytest('-vv', '--doctest-cython', str(module))
    result.stdout.fnmatch_lines([
        "*sqr*PASSED*",
    ])
    assert result.ret == 0


def test_wrap_cpp_ext_module(testdir):
    module = PATH.listdir(fil='wrap_cpp_ext_module*.so')[0]
    assert module.check()
    result = testdir.runpytest('-vv', '--doctest-cython', str(module))
    result.stdout.fnmatch_lines([
        "*sqr*PASSED*",
    ])
    assert result.ret == 0


def test_pure_py_module(testdir):
    module = PATH.listdir(fil='pure_py_module*.py')[0]
    assert module.check()
    result = testdir.runpytest('-vv', '--doctest-cython', str(module))
    result.stdout.fnmatch_lines([
        "*Eggs.__init__*PASSED*",
        "*Eggs.foo*PASSED*",
        "*foo*PASSED*",
    ])
    assert result.ret == 0



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

from glob import glob
from os import path

from setuptools import find_packages
from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, 'README.rst'), encoding='utf-8') as readme_file:
    long_description = readme_file.read()


setup(
    name='pytest-cython',
    version='0.1.1',
    description='A plugin for testing Cython extension modules',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Logan Page',
    author_email='page.lg@gmail.com',
    license='MIT',
    url='https://github.com/lgpage/pytest-cython',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[path.splitext(path.basename(p))[0] for p in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: https://pypi.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        # 'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        # uncomment if you test on these interpreters:
        # 'Programming Language :: Python :: Implementation :: IronPython',
        # 'Programming Language :: Python :: Implementation :: Jython',
        # 'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Software Development :: Testing',
        'Topic :: Utilities',
    ],
    keywords=[
        'pytest', 'py.test', 'cython', 'doctest',
    ],
    install_requires=[
        'pytest>=2.7.3',
    ],
    extras_require={
    },
    cmdclass={
    },
    entry_points={
        'pytest11': [
            'pytest_cython = pytest_cython.plugin',
        ],
    },
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import


if __name__ == "__main__":
    import os
    import sys
    import glob

    from setuptools import setup
    from setuptools import Extension

    root = os.path.dirname(__file__)
    directives = {
        'profile': True,
        'embedsignature': True,
        'linetrace': False,
    }

    # Enable code coverage for C code: we can't use CFLAGS=-coverage in
    # tox.ini, since that may mess with compiling dependencies (e.g. numpy).
    # Therefore we set SETUPPY_CFLAGS=-coverage in tox.ini and copy it to
    # CFLAGS here (after deps have been safely installed).
    macros = []
    if 'TOXENV' in os.environ and 'SETUPPY_CFLAGS' in os.environ:
        os.environ['CFLAGS'] = os.environ['SETUPPY_CFLAGS']
        if '-coverage' in os.environ['SETUPPY_CFLAGS']:
            directives['linetrace'] = True
            macros = [[('CYTHON_TRACE', '1'), ('CYTHON_TRACE_NOGIL', '1')]]


    try:
        sys.argv.remove("--use-cython")
        use_cython = True
        from Cython.Build import cythonize
        from Cython.Distutils import build_ext
    except ValueError:
        use_cython = False
        from distutils.command.build_ext import build_ext

    if 'clean' in sys.argv:
        [os.remove(x) for x in glob.glob(os.path.join(root, 'src/pypackage/*.c'))]
        [os.remove(x) for x in glob.glob(os.path.join(root, 'src/pypackage/*.so'))]
        [os.remove(x) for x in glob.glob(os.path.join(root, 'src/pypackage/*.cpp'))]

    if use_cython:
        ext_files = glob.glob(os.path.join(root, 'src/pypackage/*.pyx'))
        ext_files.extend(glob.glob(os.path.join(root, 'src/pypackage/*.py')))
    else:
        ext_files = glob.glob(os.path.join(root, 'src/pypackage/*.c'))
        ext_files.extend(glob.glob(os.path.join(root, 'src/pypackage/*.cpp')))

    extensions = []
    exclude_files = ['__init__.py']
    include_dirs = [os.path.abspath(os.path.join(root, 'src/clib'))]
    for file_ in ext_files:
        basename = os.path.basename(file_)
        if basename in exclude_files:
            continue
        pyx_file, _ = os.path.splitext(basename)
        extensions.append(Extension(
                'src.pypackage.' + pyx_file,
                [file_],
                define_macros=macros,
                include_dirs=include_dirs,
            )
        )

    if use_cython:
        extensions = cythonize(
            extensions,
            force=True,
            compiler_directives=directives,
        )


    class optional_build_ext(build_ext):
        """Allow the building of C extensions to fail."""
        def run(self):
            try:
                build_ext.run(self)
            except Exception as e:
                self._unavailable(e)
                self.extensions = []  # avoid copying missing files (it would fail).

        def _unavailable(self, e):
            print('*' * 80)
            print('''WARNING:

        An optional code optimization (C extension) could not be compiled.

        Optimizations for this package will not be available!
            ''')

            print('CAUSE:')
            print('')
            print('    ' + repr(e))
            print('*' * 80)

    setup(
        name='pytest-cython',
        version='0.1.1',
        description="Example Cython project for pytest-cython tests",
        zip_safe=False,
        cmdclass={'build_ext': optional_build_ext},
        ext_modules=extensions,
    )
